#ifndef __GRANULAR_SOUND__
#define __GRANULAR_SOUND__

/**
* Synth granulaire.
* Lui charger un fichier de base ou il pioche les grains
* utiliser setGrainParam pour modifier les params d'extraction et d'ajout des grains
*/

#include "continuous_sound.h"

#define GRAIN_SYNTHE_BUFFER_LENGTH 1.0f //On prend 1s de buffer en memoire

/**
  * Utilise plusieurs buffers :
  * un buffer qui contient le fichier de base
  * un buffer dans lequel on fait la synthse : on y ajoute les grains les un a la suite des autres
  *   en tenant compte de params de synthse. C'est un buffer tournant.
  * un buffer temporaire qui permet de crer
  */
class SoundGrain : public ContinuousSound
{
private:

	//Datas du fichier de base
	float _DureeBase; ///< Duree du buffer de base
	void * _BaseBuffer; ///< Buffer qui contient les chantillons du fichier de base
	uint8 * _PtRdFinBaseBuffer; ///< Pointeur sur la fin du buffer de base
	ALenum _Format; ///< Format du fichier de base (enum OpenAL comme AL_FORMAT_MONO16)
	ALsizei _Size; ///< Taille du fichier de base en octets
	ALfloat _Frequency; ///< Frequence d'chantillonage du fichier de base (44100 par exemple)
	int _NbPistes; ///< Nombre de pistes du fichier de base (1 mono, 2 stro)
	int _NbBits; ///< Nombre de bits par chantillon (16 ou 8, surtout utile pour affichage)
	int _TailleEchantillon; ///< Taille d'un chantillon en octets (plus utile pour les calculs)

	//Buffer pour faire la synthse (buffer tournant)
	//Ncessaire car overlap des grains 
	float _DureeBufferSyntheGrain; ///<Duree du buffer de synthse
	void * _DatasSyntheGrain; ///< Pointeur sur le dbut du buffer de synthse
	uint8 * _PtFinBufferSyntheGrain; ///< Pointeur sur la fin du buffer de synthe
	int _TailleBufferSyntheGrain; ///< Taille du buffer de synthse
	uint8 * _PtRdSyntheGrain; ///< Pointeur de lecture du buffer de synthese
	uint8 * _PtWrSyntheGrain; ///< Pointeur d'ecriture dans le buffer de synthe
	
	//Paramtres de la generation de grains
	float _PosGrainf; ///< Position de prise du grain entre 0 et 1
	float _DureeGrain; ///< Dure du grain en secondes 
	int _TailleGrain; ///< Taille du grain en octets
	int _PosGraini; ///< Position du grain en octets
	int _RandomWidth; //Taille du random quand on chope un grain
	int _SizeOverlap; //Taille d'overlapp entre les grains
	float _CrossFade; //Si on utilise un crossfade linaire ou equalpower (signal corel ou pas)

	//Pour la lecture des grains. 
	uint8 * _PtRdGrain; ///< Pointeur de lecture du grain courant
	uint8 * _PtRdFinGrain; ///< Pointeur de fin du grain courant
	int _TailleLastGrain; ///< Taille du grain courant

public :
	void loadBaseFile(char * filename)
	{
		

		//On set des params par defaut au cas ou
		setGrainParam(0.5f,0.2f,0.2f,0.05f,0.5f);
	}

	void unload()
	{
		SAFEDELETE_TAB(_DatasSyntheGrain);
		free(_BaseBuffer); _BaseBuffer = NULL; //AlutLoad* = malloc;
	}

	void setGrainParam(float posGrain, float dureeGrain, float randomPos, float partOverlap, float crossFade)
	{
		_PosGrainf = posGrain;
		_DureeGrain = dureeGrain;
		_TailleGrain = (int) ( _DureeGrain * _Frequency * _TailleEchantillon );
		_TailleGrain = (_TailleGrain/4)*4; //On s'aligne sur stereo 16 bits
		_PosGraini = (int) (_Size * _PosGrainf);
		_PosGraini = (_PosGraini / 4) * 4;  //On s'aligne sur 16 bits deux voies au cas ou 
		_RandomWidth = (int) ( randomPos * _Frequency *_TailleEchantillon );
		_SizeOverlap = (int ) ( partOverlap * _DureeGrain * _Frequency * _TailleEchantillon );
		_SizeOverlap = (_SizeOverlap/4)*4; //On aligne
		_CrossFade = crossFade;
	}


private :

	/**
	  * Phase de synthse
	  * Cette mthode va lire le buffer qui contient le fichier de base pour en extraire de grains
	  * qu'elle va copier au fur et  mesure dans le buffer de synthse
	  * Quand elle a fini la synthse, elle retourne un chantillon pris dans le buffer de synthse, 
	  * sous le pointeur de lecture, et avance ce pointeur de lecture.
	  */
	virtual float getNextSample()
	{
		return 0.0f;
	}

	
};

#endif
